@@ -1,7 +1,7 @@
 /*
  *  This file is part of gpw.
  *
- *  Copyright (C) 2013-2015 Bruce Korb, all rights reserved.
+ *  Copyright (C) 2013-2016 Bruce Korb, all rights reserved.
  *  This is free software. It is licensed for use, modification and
  *  redistribution under the terms of the GNU General Public License,
  *  version 3 or later <http://gnu.org/licenses/gpl.html>
@@ -48,12 +48,18 @@ static void
 get_pbkdf2_pw(char * buf, size_t bsz,
               char const * tag, char const * txt, char const * nam);
 
+static void
+print_pwid_header(char const * name);
+
 static void
 print_pwid_status(char const * name);
 
 static void
 select_chars(unsigned char * txtbuf);
 
+static bool
+print_one_pwid(tOptionValue const * seed_opt, char const * name);
+
 static void
 print_pwid(char const * name);
 
@@ -169,6 +175,9 @@ static void
 remove_opt(char const * txt, char const * mark, size_t m_len,
            set_opt_enum_t typ);
 
+static bool
+removed_old_opts(char const * cfg_text, char const * name, char ** mark_p);
+
 static void
 update_pwid_opts(char const * name);
 
@@ -194,6 +203,9 @@ get_seed_text(void);
 static uint32_t
 ver_str_to_number(void);
 
+static inline void
+print_new_seed(FILE * fp, char const * cfg_text);
+
 static void
 add_seed(void);
 
