@@ -1,7 +1,7 @@
 /*
  *  This file is part of gpw.
  *
- *  Copyright (C) 2013-2015 Bruce Korb, all rights reserved.
+ *  Copyright (C) 2013-2016 Bruce Korb, all rights reserved.
  *  This is free software. It is licensed for use, modification and
  *  redistribution under the terms of the GNU General Public License,
  *  version 3 or later <http://gnu.org/licenses/gpl.html>
@@ -35,22 +35,11 @@ static void
 adjust_pw(char * buf, size_t bsz, unsigned char * data, size_t d_len)
 {
     char * dta = (char *)data;
-    switch (OPT_VALUE_CCLASS & (CCLASS_NO_ALPHA | CCLASS_NO_SPECIAL)) {
-    case 0:
-    case CCLASS_NO_SPECIAL:
-        base64_encode(dta, d_len, buf, bsz);
-        buf[OPT_VALUE_LENGTH] = '\0';
-        fix_std_pw(buf);
-        break;
+    unsigned int cclass = OPT_VALUE_CCLASS & (CCLASS_NO_ALPHA | CCLASS_NO_SPECIAL);
 
-    case CCLASS_NO_ALPHA:
-        base64_encode(dta, d_len, buf, bsz);
-        buf[OPT_VALUE_LENGTH] = '\0';
-        fix_no_alpha_pw(buf);
-        break;
-
-    case CCLASS_NO_ALPHA | CCLASS_NO_SPECIAL:
-    {
+    // Check for PIN number password:
+    //
+    if (cclass == (CCLASS_NO_ALPHA | CCLASS_NO_SPECIAL)) {
         static uint32_t const bytes_per_val = 7
 #if SIZEOF_CHARP > 4
             + 10
@@ -62,8 +51,25 @@ adjust_pw(char * buf, size_t bsz, unsigned char * data, size_t d_len)
                 (uint32_t)OPT_VALUE_LENGTH, mx);
 
         fix_digit_pw(buf, (uintptr_t *)data);
+        return;
     }
+
+    base64_encode(dta, d_len, buf, bsz);
+
+    // Check for confirmation answer password
+    //
+    if (HAVE_OPT(CONFIRM)) {
+        buf[CONFIRM_LEN] = '\0';
+        fix_lower_only_pw(buf);
+        return;
     }
+
+    buf[OPT_VALUE_LENGTH] = '\0';
+
+    if (cclass == CCLASS_NO_ALPHA)
+        fix_no_alpha_pw(buf);
+    else
+        fix_std_pw(buf);
 }
 
 /**
@@ -90,6 +96,8 @@ get_dft_pw(char * buf, size_t bsz,
     sha256_process_bytes(tag, strlen(tag)+1, &ctx);
     sha256_process_bytes(txt, strlen(txt)+1, &ctx);
     sha256_process_bytes(nam, strlen(nam)+1, &ctx);
+    if (HAVE_OPT(CONFIRM))
+        sha256_process_bytes(OPT_ARG(CONFIRM), strlen(OPT_ARG(CONFIRM))+1, &ctx);
     sha256_finish_ctx(&ctx, sum.sha_buf);
 
     adjust_pw(buf, bsz, sum.sha_buf, sizeof(sum.sha_buf));
@@ -111,17 +119,24 @@ get_pbkdf2_pw(char * buf, size_t bsz,
 {
     size_t tag_len = strlen(tag) + 1;
     size_t nam_len = strlen(nam) + 1;
+    size_t cfm_len = HAVE_OPT(CONFIRM) ? (strlen(OPT_ARG(CONFIRM)) + 1) : 0;
+
     size_t hash_ln = 4 + ((bsz * 6) >> 3);
 
-    char * nam_tag = scribble_get(tag_len + nam_len);
+    char * nam_tag = scribble_get(tag_len + nam_len + cfm_len);
     char * hash_bf = scribble_get(hash_ln);
     Gc_rc rc;
 
     memcpy(nam_tag, tag, tag_len);
     memcpy(nam_tag + tag_len, nam, nam_len);
+    tag_len += nam_len;
+    if (cfm_len > 0) {
+        memcpy(nam_tag + tag_len, OPT_ARG(CONFIRM), cfm_len);
+        tag_len += cfm_len;
+    }
 
     rc = gc_pbkdf2_sha1(
-        nam_tag, tag_len + nam_len,
+        nam_tag, tag_len,
         txt, strlen(txt) + 1,
         OPT_VALUE_PBKDF2,
         hash_bf, hash_ln);
@@ -131,6 +146,14 @@ get_pbkdf2_pw(char * buf, size_t bsz,
     adjust_pw(buf, bsz, (unsigned char *)hash_bf, hash_ln);
 }
 
+static void
+print_pwid_header(char const * name)
+{
+    static char const hdr_fmt[]  = "password id '%s'%s:\n";
+    static char const is_sec[]   = " (shared password)";
+    printf(hdr_fmt, name, ENABLED_OPT(SHARED) ? is_sec : "");
+}
+
 /**
  * Print the passwords for \a name.
  * @param name  the name/id for which a password is needed
@@ -138,21 +161,20 @@ get_pbkdf2_pw(char * buf, size_t bsz,
 static void
 print_pwid_status(char const * name)
 {
-    static char const hdr_fmt[]  = "password id '%s':\n";
     static char const lstr_fmt[] = "  %-10s %s\n";
     static char const ldig_fmt[] = "  %-10s %u\n";
 
     bool have_data = false;
 
     if (HAVE_OPT(LOGIN_ID)) {
-        printf(hdr_fmt, name);
         have_data = true;
+        print_pwid_header(name);
         printf(lstr_fmt, "login-id", OPT_ARG(LOGIN_ID));
     }
 
     if (HAVE_OPT(LENGTH)) {
         if (! have_data) {
-            printf(hdr_fmt, name);
+            print_pwid_header(name);
             have_data = true;
         }
         printf(ldig_fmt, "length", (unsigned int)OPT_VALUE_LENGTH);
@@ -160,7 +182,7 @@ print_pwid_status(char const * name)
 
     if (HAVE_OPT(PBKDF2) || (OPT_VALUE_LENGTH > (MIN_BUF_LEN - 8))) {
         if (! have_data) {
-            printf(hdr_fmt, name);
+            print_pwid_header(name);
             have_data = true;
         }
         if (ENABLED_OPT(PBKDF2) || (OPT_VALUE_LENGTH > (MIN_BUF_LEN - 8)))
@@ -171,7 +193,7 @@ print_pwid_status(char const * name)
 
     if (HAVE_OPT(SPECIALS)) {
         if (! have_data) {
-            printf(hdr_fmt, name);
+            print_pwid_header(name);
             have_data = true;
         }
         printf(lstr_fmt, "spec chars", OPT_ARG(SPECIALS));
@@ -181,7 +203,7 @@ print_pwid_status(char const * name)
         char const * names;
 
         if (! have_data) {
-            printf(hdr_fmt, name);
+            print_pwid_header(name);
             have_data = true;
         }
         doOptCclass(OPTPROC_RETURN_VALNAME, &DESC(CCLASS));
@@ -230,6 +252,74 @@ select_chars(unsigned char * txtbuf)
     strcpy((char *)txtbuf, buf);
 }
 
+static bool
+print_one_pwid(tOptionValue const * seed_opt, char const * name)
+{
+    if (seed_opt->valType != OPARG_TYPE_HIERARCHY)
+        die(GNU_PW_MGR_EXIT_BAD_SEED, bad_seed);
+
+    /*
+     * Ensure that we have a reasonably current seed.
+     * If not, we ignore the seed.
+     */
+    {
+        tOptionValue const * ver = optionGetValue(seed_opt, s_ver_z);
+
+        if ((ver == NULL) || (ver->valType != OPARG_TYPE_NUMERIC)) {
+            tOptionValue const * tag = optionGetValue(seed_opt, tag_z);
+            warning_msg(too_old_z, tag->v.strVal);
+            return false;
+        }
+    }
+
+    /*
+     *  make sure that the password id setting for "shared"
+     *  matches that of our seed.
+     */
+    {
+        tOptionValue const * sec = optionGetValue(seed_opt, sec_pw_id);
+
+        if ((sec == NULL) != (! HAVE_OPT(SHARED)))
+            return false;
+    }
+
+    /*
+     * The gauntlett has been run.  Now print the password.
+     */
+    tOptionValue const * tag = optionGetValue(seed_opt, tag_z);
+    tOptionValue const * txt = optionGetValue(seed_opt, text_z);
+
+    if (  (tag->valType != OPARG_TYPE_STRING)
+       || (tag->valType != OPARG_TYPE_STRING))
+        die(GNU_PW_MGR_EXIT_BAD_SEED, bad_seed);
+
+    /*
+     * Use the PBKDF function if it is requested or if the result
+     * length exceeds what we can provide with 256 bits of hash
+     * (40 bytes).
+     */
+    /*
+     * The "txtbuf" is much larger than needed.  It gets trimmed.
+     * This way, base64encode can encode all the data,
+     */
+    size_t buf_len = (OPT_VALUE_LENGTH > (MIN_BUF_LEN - 8))
+        ? OPT_VALUE_LENGTH + 16 : MIN_BUF_LEN;
+    unsigned char * txtbuf = scribble_get(buf_len);
+
+    if (ENABLED_OPT(PBKDF2) || (OPT_VALUE_LENGTH > (MIN_BUF_LEN - 8)))
+        get_pbkdf2_pw((char *)txtbuf, buf_len,
+                      tag->v.strVal, txt->v.strVal, name);
+    else
+        get_dft_pw((char *)txtbuf, buf_len,
+                   tag->v.strVal, txt->v.strVal, name);
+
+    if (HAVE_OPT(SELECT_CHARS))
+        select_chars(txtbuf);
+
+    printf(pw_fmt, tag->v.strVal, txtbuf);
+    return true;
+}
+
 /**
  * Print the passwords for \a name.
  * @param name  the name/id for which a password is needed
@@ -237,10 +327,9 @@ select_chars(unsigned char * txtbuf)
 static void
 print_pwid(char const * name)
 {
-    size_t          buf_len = MIN_BUF_LEN;
-    unsigned char * txtbuf;
     char const *    pfx    = "";
     tOptionValue const * ov = optionFindValue(&DESC(SEED), NULL, NULL);
+    bool printed_pw = false;
 
     set_pwid_opts(name);
     if (HAVE_OPT(STATUS)) {
@@ -255,15 +344,6 @@ print_pwid(char const * name)
 
     scribble_free();
 
-    if (OPT_VALUE_LENGTH > (MIN_BUF_LEN - 8)) // > 40
-        buf_len = OPT_VALUE_LENGTH + 16;
-
-    /*
-     * The "txtbuf" is much larger than needed.  It gets trimmed.
-     * This way, base64encode can encode all the data,
-     */
-    txtbuf = scribble_get(buf_len);
-
     if (! HAVE_OPT(LOGIN_ID)) {
         if (! HAVE_OPT(NO_HEADER))
             fputs(hdr_str, stdout);
@@ -283,43 +363,13 @@ print_pwid(char const * name)
      * For each <seed> value in the config file, print a password.
      */
     do  {
-        tOptionValue const * tag, * txt, * s_ver;
+        printed_pw |= print_one_pwid(ov, name);
+        ov = optionFindNextValue(&DESC(SEED), ov, NULL, NULL);
+    } while (ov != NULL);
 
-        if (ov->valType != OPARG_TYPE_HIERARCHY)
-            die(GNU_PW_MGR_EXIT_BAD_SEED, bad_seed);
-
-        tag   = optionGetValue(ov, tag_z);
-        txt   = optionGetValue(ov, text_z);
-        s_ver = optionGetValue(ov, s_ver_z);
-
-        if (  (tag->valType != OPARG_TYPE_STRING)
-           || (tag->valType != OPARG_TYPE_STRING))
-            die(GNU_PW_MGR_EXIT_BAD_SEED, bad_seed);
-
-        if ((s_ver == NULL) || (s_ver->valType != OPARG_TYPE_NUMERIC)) {
-            warning_msg(too_old_z, tag);
-            continue;
-        }
-
-        /*
-         * Use the PBKDF function if it is requested or if the result
-         * length exceeds what we can provide with 256 bits of hash
-         * (40 bytes).
-         */
-        if (ENABLED_OPT(PBKDF2) || (OPT_VALUE_LENGTH > (MIN_BUF_LEN - 8)))
-            get_pbkdf2_pw((char *)txtbuf, buf_len,
-                          tag->v.strVal, txt->v.strVal, name);
-        else
-            get_dft_pw((char *)txtbuf, buf_len,
-                       tag->v.strVal, txt->v.strVal, name);
-
-        if (HAVE_OPT(SELECT_CHARS))
-            select_chars(txtbuf);
-
-        printf(pw_fmt, tag->v.strVal, txtbuf);
-
-    } while (ov = optionFindNextValue(&DESC(SEED), ov, NULL, NULL),
-             ov != NULL);
+    if (! printed_pw)
+        die(GNU_PW_MGR_EXIT_NO_SEED, no_passwords,
+            ENABLED_OPT(SHARED) ? sec_pw_type : "");
 
     update_pwid_opts(name);
 }
@@ -453,10 +503,12 @@ main(int argc, char ** argv)
         die(GNU_PW_MGR_EXIT_INVALID, had_load_opts);
 
     /*
-     * There are three operational modes: add a new seed, remove an old seed
-     * and print a password.  If we have any operands, we must be printing a
-     * password.  We must have found a seed option in the config file and
-     * we cannot have a --text or --tag option specified.
+     * There are four operational modes:
+     *
+     * 1) command line operands signify printing a password, otherwise
+     * 2) not having a --tag option says to read a password id from stdin, else
+     * 3) not having --text option says to remove a seed, else
+     * 4) add a new password seed using --tag and --text
      */
     if (argc > 0) {
         char const * arg;
@@ -475,17 +527,18 @@ main(int argc, char ** argv)
         print_pwid(arg);
 
     } else if (! HAVE_OPT(TAG)) {
-        /*
-         * libopts has ensured that we do not have --text, so we must
-         * be reading in a password id from standard input
-         */
         stdin_pwid();
 
-    } else if (HAVE_OPT(TEXT))
+    } else if (HAVE_OPT(TEXT)) {
+        if (HAVE_OPT(SHARED) && ! ENABLED_OPT(SHARED))
+            usage_message(disable_second);
         add_seed();
 
-    else
+    } else {
+        if (HAVE_OPT(SHARED))
+            usage_message(shared_removal);
         rm_seed();
+    }
 
     secure_cfg_file();
 
